#!/usr/bin/perl
package FestVox::ScriptLang;

BEGIN {
  @FestVox::ScriptLang::ISA = qw(Exporter); 
}

@EXPORT = qw/color hold play record script synth display/;

use strict;
use FestVox::PointyClicky;

sub color {
  my $self;
  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  }

  $self->singleton->wait(sub { $self->tk->{mw}->update });

  $self->singleton->active(1);
  my $color = shift || $self->bgcolor;
#  $self->status("  color('$color')", 'black');
  $self->bgcolor_set($color);
  $self->singleton->active(0);
  $color ;
}

sub hold {
  my $self;
  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  }

  $self->singleton->wait(sub { $self->tk->{mw}->update });

  $self->singleton->active(1);
  my $param = shift;
  if (not defined $param) {
    $param = $self->time_pause;
  }
  $self->status("  hold($param)", 'black');
  select(undef,undef,undef,$param);
  $self->tk->{mw}->update;
  $self->singleton->active(0);
}

sub play {  
  my $self;
  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  }

  $self->singleton->wait(sub { $self->tk->{mw}->update });

  my @files = @_;

  my $ap = $self->play_command;
  if (not $ap) {
    $self->status("can't play: no play_command set", "darkred");
    return undef;
  }

  my $type;
  my $tk = $self->tk;

  my $i;
  if (not @files) {
    $i = $self->iter_index;
    if (not defined $i) {
      my @s = $tk->{hlist}->selectionGet;
      $i = shift @s if @s;
    }

    if (not defined $i) {
      $self->status("play: nothing selected", "darkred");
      return undef;
    }

    my $track = $self->get_track($i);
    return undef unless defined $track;

    my $f ;
    my $wf  = $track->recorded_wav_file;
    my $pwf = $track->prompt_wav_file;

    if (defined $type) {
      $f = ( $type eq 'recorded' ) ? $wf : $pwf;
    } else {
      my $pp = $self->play_plan;
      if ($pp eq 'prompt') {
	# play prompt only
	$f = $pwf;
	$type = 'prompt';
      } elsif ($pp eq 'recorded') {
	# play recorded waveform only
	$f = $wf;
	$type = 'recorded';
      } elsif ($pp eq 'recorded | prompt') {
	# play recorded if exists, otherwise prompt
	if (-e $wf) {
	  $type = 'recorded';
	  $f = $wf;
	} elsif (-e $pwf) {
	  $type = 'prompt';
	  $f = $pwf;
	} else {
	  $type = 'none';
	  $self->status("nothing to play", "darkred");
	  return undef;
	} 
      } elsif ($pp eq 'prompt & recorded') {
      } else {
	$self->status("unknown Play strategy: $pp; reverting");
	$self->play_plan('recorded | prompt');
      }
    }
    push @files, $f if -e $f;
  }

  for my $file (@files) {
    my $c = sprintf($ap, $file);
    if (not $file) {
      $self->status("empty filename", "darkred");
      next ;
    }

    if (not -e $file) {
      $self->status("no such file as $file", "darkred");
    } elsif (-s $file) {
      $self->status("  play('$file')", "blue");
      $self->singleton->start
	  ('command' => $c, 
	   'context' => 'system',
	   'type' => "play", 
	   'async' => 1);
#	   'async' => 0);
    } else {
      $self->status("skipping empty $file", "darkred");
    }
  }
}

sub record {
  my $self;
  my $arg = shift;

  warn "Arg to record not supported" if defined $arg;

  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  }

  $self->singleton->wait(sub { $self->tk->{mw}->update });

  my @s = @_;
  my $tk = $self->tk;

  my $i = shift @s;
  if (not defined $i) {
    $i = $self->iter_index;
    if (not defined $i) {
      my @s = $tk->{hlist}->selectionGet;
      $i = shift @s;
    }
  }

  if (not defined $i) {
    $self->status("record: nothing selected", "darkred");
    return undef;
  }

  my $ar = $self->record_command;
  if (not $ar) {
    $self->status("no record_command set", "darkred");
    return undef;
  }

  my $track = $self->get_track($i);
  return undef unless defined $track;

  my $file = $track->recorded_wav_file;
  my $name = $track->basename;
  $self->recording_file($file);

  my $rt;
  if (my $style = $self->time_style) {
    if ($style eq 'fixed') {
      $rt = $self->record_duration;
    } elsif ($style eq 'prompt') {
      $rt = $track->guess_duration;
    } elsif ($style eq 'track') {
      $rt = $track->duration;
    } else {
      warn "other time_style $style";
    }
  }

  my $c = $ar;
  if ($rt) {
    # magically add recording duration
    $c =~ s/^(na_record )/$1 -time $rt /;
    $c =~ s/^(arecord )/$1 -d $rt /;
  }
  $c = sprintf($c, $file);

  my (@buttons, $text);
  @buttons = qw/Record Cancel/;
  push @buttons, 'Stop' if $self->iterating;

  my $r;

  if ($self->ask_on_record_over and -e $file) {
    $text = "Record over $name?";
  } elsif ($self->ask_on_record) {
    $text = "Record $name?";
  } else {
    # there's nothing to ask about
    $r = 'Record';
  }

  if ($text) {
    my $dialog = $tk->{mw}->Dialog
	(-text => $text,
	 -bitmap => 'question', 
	 -title => 'Record', 
	 -default_button => 'Record',
	 -buttons => \@buttons
	 ); 
    $r = $dialog->Show();
  }

  if ($r eq 'Record') {
    $self->status("  record('$name')", "blue");
    unlink $file;

    # Run the 'pre-record' command if set.
    my $pre_r = $self->pre_record_command;
    if ($pre_r) {
	    $self->singleton->start
		    ('command' => $pre_r, 
		     'context' => 'system',
		     'type' => "pre_record", 
		     'async' => 0); # Must be syncronous and finished before recording.
    }

    $self->singleton->start
	('command' => $c, 
	 'context' => 'system',
	 'type' => "record", 
	 'async' => 1);

    # Run the 'post-record' command if set.
    my $post_r = $self->post_record_command;
    if ($post_r) {
	    $self->singleton->start
		    ('command' => $post_r, 
		     'context' => 'system',
		     'type' => "post_record", 
		     'async' => 0); # There is at best no need to be async.
    }
  } elsif ($r eq 'Cancel') {
    $self->status("Recording cancelled", "black");
    return undef;
  } elsif ($r eq 'Stop') {
    $self->status("Iteration stopped.", "black");
    $self->stop_iterating;
    return undef;
  }
}

sub script {
  my $self;
  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  }
  my $script = shift;

  $self->singleton->wait(sub { $self->tk->{mw}->update });

  my $i = $self->tk->{hlist}->selectionGet;

  if (not -T $script) {
    $self->status("$script isn't an ascii file", 'darkred');
    return undef;
  }
  do $script;

}

sub synth {
  my $self;
  my $text = shift;
  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  } 

  $self->singleton->wait(sub { $self->tk->{mw}->update });
  
  my $festival = $self->festival;
  if (not defined $festival) {
    $festival = FestVox::Festival->new();
    my $success = $festival->start;
    if ($success) {
      $self->festival($festival);
    } else {
      $self->status("Failed to connect to festival server", 'darkred');
      return 0;
    }
  }

  $self->singleton->active(1);
  my $i = $self->iter_index;
  my $track = $self->get_track($i);
  return 0 unless defined $track;

  $text = $track->content unless defined $text;
  if ($text) {
    $self->status("  synth($text)", 'black');
    $self->festival->say($text);
  } else {
    $self->status("synth: no text given", 'darkred');
  }
  $self->singleton->active(0);

  undef;
}

sub display {
  my $self;
  my $text = shift;

  if (not defined ($self = $FestVox::PointyClicky::Instance)) {
    warn "No top-level FestVox::PointyClicky instance\n";
    return undef;
  } 

#  $self->singleton->wait(sub { $self->tk->{mw}->update });

  my $tk = $self->tk;

  my @sel = $tk->{hlist}->selectionGet;
  return undef unless defined $text or @sel;

  my $index = shift @sel;
  my $track = $self->get_track($index);
  return undef unless defined $track;

  $text = $track->content unless defined $text;
  my $tl = $self->text_lock;

  my $curtext = $tk->{text}->get("1.0", 'end');
#  $self->status(" display($text)", 'black');
  if ($text ne $curtext) {
    $self->set_text_lock(0) if $tl;
    $tk->{text}->delete("1.0", 'end');
    $tk->{text}->insert('end', $text);
  }
  $self->set_text_lock($tl) if $tl;
}

1;

__END__

=head1 NAME

FestVox::ScriptLang - Routines for FestVox scripting

=head1 SYNOPSIS

    display; play; 
    color("yellow"); hold(0.75);
    color("orange"); record; color;

=head1 DESCRIPTION

FestVox::ScriptLang exports a number of simple routines
that allow for easy scripting in FestVox::PointyClicky.

While these routines are Class methods, rather than
Instance methods, they exploit the fact that there
is a unique FestVox::PointyClicky object, and look
at FestVox::PointyClicky::Instance to find the 
object to use as C<$self>.  This is so that we can
get the syntactic sugar of bare words being interpreted
as perl routines, while in effect doing object method
calls.

=head1 ROUTINES

By default, the routines will act on the current
iteration selection if given no arguments.  Thus,
if a number of atoms are selected using in
C<pointyclicky>, and the routines are called, they
will use the current object by default.  Otherwise,
if given explicit arguments, they will use those
instead.

Each routine will wait until there is no active 
Singleton to execute so that the prior command
will finish in an iteration sequence.

=over 2

=item color(?color?)

Change the background panes (except the Text 
field) to the give color, or as given in the
FestVox::PointyClicky variable, C<bgcolor>,
if no argument is given.

=item display(?text?)

Shows the default item in the text window.  If the 
optional argument is supplied, it is taken as the
text to display.

=item hold(?time?)

Pause for the given time, or the default time,
as given in the FestVox::PointyClicky variable,
C<time_pause>.

=item play(?file?)

Play one of the audio files associated with the current
item.  This may either be the prompt or the recorded
file, depending on the C<play_plan> option of 
FestVox::PointyClicky.  The way C<play> actually carries
out the playing depends on the C<audio_play> variable.

If given an optional list of files, C<play> will 
play those files instead of the default one for the
current item.

=item record(?file?)

Record the audio files associated with the current
item.  The way C<record> actually carries out the playing 
depends on the FestVox::PointyClicky variable C<audio_record>.

If given an optional file name as an argument, C<record> will 
record that instead of the default one for the current item.

=item display(?text?)

Shows the default item in the text window.  If the 
optional argument is supplied, it is taken as the
text to display. Note: C<display> does NOT wait for the
prior Singleton to complete; you will have to 
deliberately call wait() on the task if you want it
to wait.

=item script(filename)

Execute the Perl program named in C<filename>.

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

pointyclicky, FestVox::PointyClicky, FestVox::ScriptLang,
FestVox::InitMap, FestVox::Track, FestVox::Singleton, 
FestVox::Task,  FestVox::Festival, FestVox::MethodMapper,
perl(1).

 http://www.festvox.org/
 http://www.cstr.ed.ac.uk/projects/festival.html
 http://www.speech.cs.cmu.edu/festival/

=cut

