package FestVox::Singleton;
use FestVox::MethodMapper;
use FestVox::Task;

BEGIN {
  @FestVox::Singleton::ISA = qw(FestVox::Task Exporter AutoLoader); 
  %FestVox::Singleton::mapping = %FestVox::Task::mapping;
}

sub new {
  my $class = shift;
  my $self  = FestVox::Task->new(@_);
  bless $self, $class;
  $self;
}

sub start {
  my $self = shift;
  $self->set(@_);
  $self->stop if $self->active;
  $self->SUPER::start;
}

sub AUTOLOAD {
  my $self = shift ;

  no strict 'vars';

  $method = $AUTOLOAD;
  $method =~ s/^.*:://;

  if (@_) {
    $self->SUPER::set($method => $_[0]);
  } else {
    my ($p, $file, $line) = caller;
    return $self->SUPER::get($method, $file, $line);
  }
}


1;

__END__

=head1 NAME

FestVox::Singleton - Singleton FestVox::Task Manager

=head1 SYNOPSIS

    use FestVox::Singleton;
    my $singleton = FestVox::Singleton->new
           ('callback' => sub { $self->cback(@_) });
    ...

    # fork off the subprocess if we're allowed to
    $singleton->start('command' => 'echo "hello world"'); 

=head1 DESCRIPTION

Stops any currently executing FestVox::Task (if there is one)
when a new one is given, and starts the new one.

Since only one Task can be active, Singleton is derived
from Task itself, rather than being an aggregrate class.
Other agenda mechanisms are not currently supported.

The only method that Singleton overrides from Task is:

=over 4

=item B<start(@args)> 

Stop any existing Task and post the new one. <@args> are passed to Task::start.

=back

=head1 AUTHOR

Copyright (c) 2000 Kevin A. Lenzo and Alan W Black, Carnegie 
Mellon Unversity.

=head1 SEE ALSO

pointyclicky, FestVox::PointyClicky, FestVox::ScriptLang,
FestVox::InitMap, FestVox::Track, FestVox::Singleton, 
FestVox::Task,  FestVox::Festival, FestVox::MethodMapper,
perl(1).

 http://www.festvox.org/
 http://www.cstr.ed.ac.uk/projects/festival.html
 http://www.speech.cs.cmu.edu/festival/

=cut


